#!/bin/sh
# Copyright (C) James Budiono 2013, 2014
# License: GNU GPL Version 3 or later.
#
# Fatdog RootFS final adjustment script
# Run this last - just before makesfs
# . = ROOT

### configuration
SPOT_HOME=./root/spot
MERGE_FIREFOX_SEAMONKEY= # 'yes' to do it, or leave blank to disable
XDG_DEFAULT_LIST=usr/share/applications/defaults.list
DISABLE_USR="s@^usr/@@"
SETUID_BINARIES="
bin/busybox-cron
bin/busybox-login
bin/passwd
bin/su
bin/mount-FULL
bin/umount-FULL
sbin/crond
usr/lib/openssh/ssh-keysign
usr/lib/dbus-1.0/dbus-daemon-launch-helper
usr/bin/gtksu
usr/bin/Xorg
usr/bin/screen-4.0.3
usr/bin/screen-4.2.1
usr/bin/fusermount
usr/bin/fbterm
"

### helpers ###

# $1-path
create_bbwrapper() {
	local appletname=${1##*/}
	rm $1
	cat > $1 << EOF
#!/bin/ash
exec busybox $appletname "\$@"
EOF
	chmod +x $1
}

# $1-old name $2-new name
rename_binary() {
	if readlink -f $1 | grep -q busybox; then
		create_bbwrapper $1
	fi
	mv $1 $2
}

### main ###
# 1. remove fossil placeholders
echo cleanup ...
find . -name placeholder_for_fossil -delete

# 2. dirs we can't create earlier: dev proc sys archive tmp
echo dev proc sys tmp archive dir...
mkdir -p dev proc sys archive tmp
chown root:users archive
chmod 1770 archive
chmod 1777 tmp var/tmp

# 3. Create busybox symlinks, if needed
if [ -e bin/busybox-applet-list ]; then
	sed "$DISABLE_USR" < bin/busybox-applet-list | while read -r p; do 
		if ! [ -e $p ]; then 
			ln -s ../bin/busybox $p # let it fail when something else already exists
		fi
	done
fi

# 4. default shell is bash
echo symlinks...
ln -sf bash bin/sh 

# 5. mtab
ln -sf /proc/mounts etc/mtab

# 6. adjust mount & umount
rename_binary bin/mount bin/mount-FULL
rename_binary bin/umount bin/umount-FULL
ln -sf fatdog-mount-wrapper.sh bin/mount
ln -sf fatdog-mount-wrapper.sh bin/umount

# 7. adjust losetup & udevd
rename_binary sbin/losetup sbin/losetup-FULL
ln -sf /bin/busybox sbin/losetup
ln -sf /lib/udev/udevd sbin/udevd

# 8. xauth & sftp-server for dropbear
mkdir -p usr/bin/X11 usr/libexec
ln -sf /usr/bin/xauth usr/bin/X11/xauth
ln -sf /usr/lib/openssh/sftp-server usr/libexec/sftp-server

# 9. rgb.txt for older programs
mkdir -p usr/lib/X11
ln -sf /usr/share/X11/rgb.txt usr/lib/X11/rgb.txt

# 10. override initrd binaries (not needed anymore)
#ln -sf /usr/bin/awk bin/awk
#ln -sf /usr/bin/du bin/du
#ln -sf /usr/bin/echo bin/echo
#ln -sf /usr/bin/true bin/true

# 11. permissions
echo setting permissions ...
find . -type f -print0 | xargs -0 chmod -s # turn off all setuid 
chown -R spot:spot $SPOT_HOME # make sure spot owns everything in spot
chmod 0711 root home         # restrict access to root and home
chmod +s $SETUID_BINARIES 2> /dev/null
# cups - ensure backend and filters have the right permissions
chown root:root usr/lib/cups/backend/* usr/lib/cups/filter/* \
                usr/lib/cups/backend   usr/lib/cups/filter
chmod 0755 usr/lib/cups/backend usr/lib/cups/filter usr/lib/cups/filter/*
chmod 0500 usr/lib/cups/backend/* # otherwise won't run as root
chown -R webuser:webgroup var/lib/www # default webroot for mongoose
chmod 0640 /etc/shadow # make sure it is not available to anyone
chmod 0600 /etc/gtksudoer # must be this otherwise it won't work

# 12. symlink optional, missing, and startup programs 
[ -e usr/bin/glipper-lite ] && ln -sf /usr/bin/glipper-lite etc/xdg/Startup
[ -e usr/bin/sven ] && ln -sf /usr/bin/sven etc/xdg/Startup
[ -e usr/bin/retrovol ] && chmod +x etc/xdg/Startup/retrovol
[ -e usr/bin/autochown ] && chmod +x etc/init.d/91-autochownd
[ -e usr/sbin/wpa_gui ] && chmod +x etc/xdg/Startup/WpaGui
[ ! -e usr/bin/xterm -a -e usr/bin/urxvt ] && ln -sf urxvt usr/bin/xterm
[ ! -e usr/bin/rxvt  -a -e usr/bin/urxvt ] && ln -sf urxvt usr/bin/rxvt
#[ -e usr/sbin/bluetoothd ] && chmod +x etc/init.d/85-bluetooth
if [ -e usr/bin/gtkdialog ]; then # gtkdialog is main binary now
	! [ -e usr/bin/gtkdialog3 ] && ln -sf gtkdialog usr/bin/gtkdialog3
	! [ -e usr/bin/gtkdialog4 ] && ln -sf gtkdialog usr/bin/gtkdialog4
fi

# 13. replace the "-" in SendTo items with spaces
find etc/xdg/rox.sourceforge.net/SendTo/ ! -type d | while read -r p; do
	file="${p##*/}" dir="${p%/$file}"
	case "$file" in
		*-*) mv "$p" "$dir/$(echo "$file" | tr '-' ' ')" ;;
	esac
done

# 14. Update important cache files (e.g. mime-database, font.dir/font.scale)
update-mime-database usr/share/mime
for p in usr/share/fonts/X11/*; do # compensate for packages which fail to update fonts.dir
	mkfontscale "$p"
	mkfontdir "$p"
done
rm -f usr/share/info/dir
for p in usr/share/info/*info; do # compensate for packages which fails
	install-info --dir-file=usr/share/info/dir $p
done
chroot . /usr/bin/gtk-query-immodules-2.0 --update-cache
glib-compile-schemas ./usr/share/glib-2.0/schemas

# 15. GTK/freedesktop default programs
! [ -e $XDG_DEFAULT_LIST ] && echo "[Default Applications]" > $XDG_DEFAULT_LIST
>> $XDG_DEFAULT_LIST cat << "EOF"
x-scheme-handler/http=default-browser.desktop
x-scheme-handler/https=default-browser.desktop
EOF

# 16. Remove bad libraries (e.g. vdpau for nouveau)
rm -f usr/lib/vdpau/*nouveau*

# 17. make firefox re-use gecko libs from seamonkey
# (both must be installed, and they must use identical gecko version)
if [ "$MERGE_FIREFOX_SEAMONKEY" ] && [ -e usr/lib/firefox ] && [ -e usr/lib/seamonkey ]; then
	SM=$(readlink usr/lib/seamonkey) # link to actual seamonkey dir
	rm usr/lib/firefox/*.so usr/lib/firefox/*.chk
	for p in $(ls usr/lib/seamonkey); do
		case $p in
			*.so|*.chk) ln -sf /usr/lib/$SM/$p usr/lib/firefox/$p
		esac
	done	
fi

# 18. run i18n on desktop files (from L18L)
[ -x usr/sbin/update-desktop-files-i18n.sh ] &&
usr/sbin/update-desktop-files-i18n.sh usr/share/l18l/desktop/my_desktop_infos_*

# done
#rm $0 # self-destruct
