#!/bin/ash
# set default soundcard for user
#
# Copyright (C) James Budiono 2013
# License: GPL Version 3 or later
#
# $1-special command

### configuration
APPTITLE="Set Default Soundcard"
ALSADEV_PREFIX=${ALSADEV_PREFIX:-fd}
ASOUNDRC=${ASOUNDRC:-$HOME/.asoundrc}	# it can easily be /etc/asound.conf for system-wide settings

### run-time variables
SOUNDCARDS=
BT_DEVICES=
DMIX_IPC_KEY=
DSNOOP_IPC_KEY=

### returns SOUNDCARDS, BT_DEVICES, DMIX_IPC_KEY
detect_hardware() {
	SOUNDCARDS=$(aplay -l | awk '$1~/card/ { print "\"" $0 "\" " "\"" $0 "\"" }')
	BT_DEVICES=$(awk 'BEGIN { print "default Default" } { print "\"" $0 "\" " "\"" $0 "\"" } ' /var/lib/bluetooth/*/names)
	DMIX_IPC_KEY=$(od -An -tu4 -N3 /dev/urandom)
	DSNOOP_IPC_KEY=$(od -An -tu4 -N3 /dev/urandom)
	
	if pidof bluetoothd > /dev/null; then
		SOUNDCARDS="$SOUNDCARDS bluetooth \"Bluetooth devices\""
	fi
}


########### main ############
# 0. Special command (running as root)
case $1 in
	enable-radeon-audio)
		if Xdialog --title "$APPTITLE" --yesno "Enable sound over Radeon HDMI?" 0 0; then
			echo "options radeon audio=1" > /etc/modprobe.d/radeon-audio.conf
		else
			rm /etc/modprobe.d/radeon-audio.conf			
		fi
		exit
		;;
esac

# 1. Choose card
detect_hardware
if ! CARD=$(eval Xdialog --title \"$APPTITLE\" --stdout --no-tags --menubox \"Choose card to be made as default\" 10 100 5 $SOUNDCARDS); then
	CARD=cancel
fi

# 2. Choose card option (bluetooth = address)
case "$CARD" in
	cancel) ;; # do nothing if cancelled
	bluetooth) 
		if ! BT_DEV=$(eval Xdialog --title \"$APPTITLE\" --stdout --no-tags --menubox \"Choose bluetooth devices\" 10 100 5 $BT_DEVICES); then
			CARD=cancel
		else
			case "$BT_DEV" in 
				default) HW="{ type bluetooth }" ;;
				*) HW="{ type bluetooth device ${BT_DEV%% *} profile \"auto\" }" ;;
			esac
		fi
		;;
	*)	HW=$(echo $CARD | sed 's/card \([0-9]*\):.*device \([0-9]*\):.*/{ type hw card \1 device \2  }/')
		HWMIXER=$(echo $CARD | sed 's/card \([0-9]*\):.*/{ type hw card \1 }/')
		;;
esac

# 3. Choose options (plug, dmix, equal, etc)
if [ "$CARD" != "cancel" ]; then
	# prepare available options
	AVAIL_OPTIONS="plug     \"Sample rate converter\" on \
	               equal    \"Equaliser\" off \
	               softvol  \"Software volume control\" off"
	if [ "$CARD" != bluetooth ]; then
		AVAIL_OPTIONS="$AVAIL_OPTIONS \
		               dmix     \"Multiple applications can output at the same time\" on \
		               dsnoop   \"Multiple applications can record at the same time\" off \
		               force48k \"Force hardware output rate at 48 kHz\" off \
		               force44k \"Force hardware output rate at 44.1 kHz\" off"
	fi
	AVAIL_OPTIONS="$AVAIL_OPTIONS \
		           vdownmix \"Downmix multi-channel to stereo (doesnt work with equaliser)\" off"
	
	# choose options
	if ! OPTIONS=$(eval Xdialog --title \"$APPTITLE\" --stdout --no-tags --separator \" \" --checklist \"Use the following plugins\" 20 100 5 $AVAIL_OPTIONS); then
		CARD=cancel
	fi
	OPTIONS="$OPTIONS hw"	
fi

# 4. UI done, now do the heavy lifting
#echo $CARD
#echo $BT_DEV
#echo $OPTIONS
if [ "$CARD" != "cancel" ]; then
	# first pass, pre-process common options
	DEVICES="" asym_done=""
	for p in $OPTIONS; do
		case $p in
			force48k) HW="${HW% \}} rate 48000 }" ;; #$(echo "$HW" | sed 's/}/ rate 48000 }/') ;;
			force44k) HW="${HW% \}} rate 44100 }" ;; #$(echo "$HW" | sed 's/}/ rate 44100 }/') ;;
			dmix)     [ -z $asym_done ] && DEVICES="$DEVICES asym" && asym_done=1
			          DEVICES="$DEVICES $p" ;;
			dsnoop)   [ -z $asym_done ] && DEVICES="$DEVICES asym" && asym_done=1
			          DEVICES="$DEVICES $p" ;;
			*)        DEVICES="$DEVICES $p" ;;
		esac	
	done
	
	# second pass, generate the actual asoundrc file
	#echo $DEVICES
	{		
		set -- $DEVICES
		echo "pcm.!default" $ALSADEV_PREFIX$1
		[ "$HWMIXER" ] && echo "ctl.!default $HWMIXER"
		while [ $1 ]; do
			case $1 in
				hw)		echo "pcm.$ALSADEV_PREFIX$1 $HW" ;;		
				plug)	echo "pcm.$ALSADEV_PREFIX$1 plug:$ALSADEV_PREFIX$2" ;;
				equal)	echo "pcm.$ALSADEV_PREFIX$1 { type equal slave.pcm plug:$ALSADEV_PREFIX$2 } " 
						echo "ctl.equal { type equal }"
						;;

				softvol) cat << EOF
pcm.$ALSADEV_PREFIX$1 {
    type            softvol
    slave.pcm       $ALSADEV_PREFIX$2
    control.name    "SoftPCM"
    control.card    0
}
EOF
						;;

				vdownmix)
				        playback_pcm=${ALSADEV_PREFIX}hw
						case "$DEVICES" in
							*dmix*) playback_pcm=${ALSADEV_PREFIX}dmix
						esac
						echo "pcm.$ALSADEV_PREFIX$1 { type vdownmix slave.pcm $playback_pcm }"
						;;

				asym)	playback_pcm=${ALSADEV_PREFIX}hw
						capture_pcm=${ALSADEV_PREFIX}hw
						case "$DEVICES" in
							*dmix*)   playback_pcm=${ALSADEV_PREFIX}dmix  ;;
						esac
						case "$DEVICES" in
							*vdownmix*) playback_pcm=${ALSADEV_PREFIX}vdownmix  ;;						
						esac
						case "$DEVICES" in
							*dsnoop*) capture_pcm=${ALSADEV_PREFIX}dsnoop ;;
						esac
						cat << EOF
pcm.$ALSADEV_PREFIX$1 { 
		type asym 
		playback.pcm {
			@func getenv
			vars [ ALSA_PCMOUT ]
			default "$playback_pcm"
		}
		capture.pcm {
			@func getenv
			vars [ ALSA_PCMIN ]
			default "$capture_pcm"
		}
}
EOF
						;;
						
				dsnoop) cat << EOF
pcm.$ALSADEV_PREFIX$1 {
	type dsnoop
	ipc_key $DSNOOP_IPC_KEY
	ipc_gid audio
	ipc_perm 0660
	slave.pcm ${ALSADEV_PREFIX}hw
}
EOF
;;
				dmix)
					cat << EOF
pcm.$ALSADEV_PREFIX$1 { 
	type dmix 
	ipc_key $DMIX_IPC_KEY
	ipc_gid audio
	ipc_perm 0660 
	slave {
		pcm ${ALSADEV_PREFIX}hw
		period_time 0
		period_size 2048
		buffer_size 32768		
	}
}
EOF
;;
			esac
			shift
		done
cat << "EOF"

# for use plug/plughw overrides
pcm.vdownmix 
{
	@args [ SLAVE ]
	@args.SLAVE {
		type string
	}
	type vdownmix
	slave.pcm $SLAVE
}
EOF
	} > $ASOUNDRC
	[ $(id -u) -eq 0 ] && cp $ASOUNDRC /root/spot # whatever we do for root we also do for spot
	
	# offer to enable radeon hdmi
	if lsmod | grep -q radeon && echo "$CARD" | grep -q HDMI; then
		[ $(id -u) -ne 0 ] && gtksu "Enable radeon audio over HDMI" "$0" enable-radeon-audio ||
		"$0" enable-radeon-audio
	fi
else
	Xdialog --title "$APPTITLE" --infobox "Operation cancelled. Nothing was changed." 0 0 10000
fi
