#!/bin/ash
# FatdogArm/Fatdog64 package build script
# Copyright (C) James Budiono, 2013, 2014, 2015
# License: GNU GPL Version 3 or later
#
# This is the version of build-pkg that will build a single package
# from *within* FatdogArm/Fatdog64.
#
# The usual build pre-requisites (toolchain, make, bison, etc) 
# must alread be installed.
#
# $1-pkgname 

[ $(id -u) -ne 0 ] && exit # must run as root
export LANG=C # make it run a bit faster

### configuration settings (overridable)
[ $BOOTSTATE_PATH ] && [ -e $BOOTSTATE_PATH ] && 
. $BOOTSTATE_PATH # AUFS_ROOT_ID
BUILD_DIR_PREFIX=${BUILD_DIR_PREFIX:-/tmp}
BUILD_DIR=$(mktemp -d -p $BUILD_DIR_PREFIX build-pkg.XXXXXXXX)
CHROOT_DIR=/tmp/chroot-${BUILD_DIR##*/}

PKG=${1##*/}
PKG_TYPE=${PKG_TYPE:-tbz}
PKG_DIR=${PKG_DIR:-localpkg}
SRC_DIR=$(readlink -f ${SRC_DIR:-src})
PATCHES_DIR=$(readlink -f ${PATCHES_DIR:-patches})
OUTPUT_DIR=$(readlink -f ${OUTPUT_DIR:-output})
#KEEP_DIR=1
PKG_COPY=${PKG_COPY:-usr/src/pkgbuild/pkg} # keep a copy of the pkg here
PKG_COPY=${PKG_COPY#/} # make sure not absolute path
MAKEPKG=makepkg

###
trap 'cleanup; exit $RESULT' INT TERM HUP 0
CLEANED="" RESULT=1
cleanup() {
	[ $CLEANED ] && return; CLEANED=yes
	if mountpoint -q $CHROOT_DIR; then
		busybox fuser -m $CHROOT_DIR | xargs kill 2> /dev/null
		sleep 1
		busybox fuser -m $CHROOT_DIR | xargs kill -9 2> /dev/null
		for p in dev/pts dev sys proc /; do
			umount $CHROOT_DIR/$p
		done
	fi
	rmdir $CHROOT_DIR
	[ -z $KEEP_DIR ] && ! [ -e $BUILD_DIR/keep ] && 
	rm -r $BUILD_DIR
	echo done
}

###
init() {
	# clean slate
	PKG_NOMAKEPKG= PKG_NOBUILD= PKG_NOSTRIP=

	local robranches
	[ -z "$AUFS_ROOT_ID" ] && AUFS_ROOT_ID=$(
awk '{ if ($2 == "/" && $3 == "aufs") { match($4,/si=[0-9a-f]*/); print "si_" substr($4,RSTART+3,RLENGTH-3) } }' \
/proc/mounts | tail -n 1
)
	if [ $AUFS_ROOT_ID ]; then
		robranches=$(ls -v /sys/fs/aufs/$AUFS_ROOT_ID/br[0-9]* | xargs sed 's|=rw|=ro|' | tr '\n' ':') 
	else
		# not using aufs - assuming we can use root as aufs branch
		robranches=/
	fi
	chmod 0755 $BUILD_DIR # mktemp always creates 0700
	mkdir -p $PKG_DIR $SRC_DIR $OUTPUT_DIR $CHROOT_DIR 
	mount -t aufs aufs -o br:$BUILD_DIR:$robranches $CHROOT_DIR
	for p in sys proc dev dev/pts; do
		mount -o bind /$p $CHROOT_DIR/$p
	done
	rm -rf $CHROOT_DIR/tmp/* # make sure tmp is clean
	[ -e functions.sh ] && . functions.sh && cp functions.sh $CHROOT_DIR/tmp
	[ -e env.sh ] && . env.sh && cp env.sh $CHROOT_DIR/tmp
}

### $PKG input/output, $PKG_FULL_PATH output
find_pkg() {
	[ -z $PKG ] && echo Please specify package to build. && return 1	

	# find the package
	#echo $PKG
	if [ -e $PKG_DIR/$PKG ]; then # exist - take it as it is
		PKG_FULL_PATH=$PKG_DIR/$PKG
	else 
		# look for it - assuming we are given only package name
		PKG_FULL_PATH=$(find $PKG_DIR -type f -name "${PKG}-[0-9]*")
		# if we can't find, look again assuming we are given prefix
		[ -z "$PKG_FULL_PATH" ] && PKG_FULL_PATH=$(find $PKG_DIR -type f -name "${PKG}*")	
	fi

	# check if we find one and only one of our package	
	case $(echo $PKG_FULL_PATH | wc -w) in
		0) echo $PKG does not exist.
		   return 1 ;;
		1) PKG=${PKG_FULL_PATH##*/}  ;;
		*) echo Multiple packages found, cannot continue:
		   echo $PKG_FULL_PATH
		   return 1 ;;
	esac
}

###
prepare() {
	find_pkg || return 1
	echo preparing $PKG
		
	# create generic build script, can be overridden if need be
	cat > $CHROOT_DIR/tmp/build.sh << EOF
#!/bin/sh
[ -e /tmp/functions.sh ] && . /tmp/functions.sh
[ -e /tmp/env.sh ] && . /tmp/env.sh
[ -e /tmp/pkg ] && . /tmp/pkg
type pkg_build > /dev/null && pkg_build || 
{ echo "Build error $PKG, dropping to temporary shell."; sh < /dev/tty > /dev/tty 2>/dev/tty; }
EOF
	chmod +x $CHROOT_DIR/tmp/build.sh
	
	# load package script, copy for later execution in chroot
	. $PKG_FULL_PATH
	cp $PKG_FULL_PATH $CHROOT_DIR/tmp/pkg
	type pkg_prepare > /dev/null && ( cd $CHROOT_DIR; pkg_prepare; )
	[ $? -ne 0 ] && echo "Failed prepare phase." && return 1
	return 0
}

###
build() {
	echo building $PKG
	[ "$PKG_NOBUILD" ] && return 0

	#chroot $CHROOT_DIR /tmp/build.sh
	LD_LIBRARY_PATH= chroot $CHROOT_DIR /bin/env -i HOME=/root USER=root TERM=$TERM \
	   PS1='\u:\w# ' PATH=/usr/bin:/usr/sbin:/bin:/sbin:/tools/bin \
	   /tmp/build.sh
	[ $? -ne 0 ] && echo "Failed build phase." && return 1
	return 0
}

###
package() {
	echo packaging $PKG
	mkdir -p $CHROOT_DIR/$PKG_COPY && cp $PKG_FULL_PATH $CHROOT_DIR/$PKG_COPY
	if type pkg_package > /dev/null; then # pkg_package is optional
		( cd $CHROOT_DIR; pkg_package; )
	fi &&
	(
		cd $BUILD_DIR
		rm -rf .wh..wh.orph .wh..wh.plnk tmp \
			root/.cache root/.ccache usr/share/info/dir root/.distcc
		find . -name ".wh.*" -delete
		[ -z "$PKG_NOSTRIP" ] && find . -type f -print0 | xargs -0 strip --strip-unneeded 2> /dev/null
		[ -z "$PKG_NOMAKEPKG" ] && ! [ -e no-makepkg ] && 
		$MAKEPKG -c n -l n $OUTPUT_DIR/$PKG.$PKG_TYPE > /dev/null 2>&1 || return 0
	)
	[ $? -ne 0 ] && echo "Failed packaging phase." && return 1
	return 0
}

### main
init
prepare && build && package; RESULT=$?
cleanup
